<?php

namespace Drupal\lbl;

use Drupal\breakpoint\BreakpointManagerInterface;
use Drupal\Component\Utility\Html;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Layout\LayoutDefault;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\Core\Layout\Icon\IconBuilderInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Template\Attribute;
use Drupal\Core\Theme\ThemeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base layout class for baumeister.
 */
class BaseLayout extends LayoutDefault implements PluginFormInterface, ContainerFactoryPluginInterface {

  /**
   * Icon builder.
   *
   * @var \Drupal\Core\Layout\Icon\IconBuilderInterface
   */
  protected $iconBuilder;

  /**
   * Icon builder.
   *
   * @var \Drupal\breakpoint\BreakpointManagerInterface
   */
  protected $breakpointManager;

  /**
   * Icon builder.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $config;

  /**
   * Icon builder.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Theme manager.
   *
   * @var \Drupal\Core\Theme\ThemeManagerInterface
   */
  protected $themeManager;

  /**
   * {@inheritdoc}
   */
  final public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    IconBuilderInterface $icon_builder,
    BreakpointManagerInterface $breakpoint_manager,
    ConfigFactoryInterface $config_factory,
    EntityTypeManagerInterface $entity_type_manager,
    ThemeManagerInterface $theme_manager
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->iconBuilder = $icon_builder;
    $this->breakpointManager = $breakpoint_manager;
    $this->entityTypeManager = $entity_type_manager;
    $this->config = $config_factory;
    $this->themeManager = $theme_manager;
    $this->iconBuilder->setHeight(25);
    $this->iconBuilder->setWidth(25);
    $this->iconBuilder->setPadding(1);
    $this->setConfiguration($configuration);
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('layout.icon_builder'),
      $container->get('breakpoint.manager'),
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('theme.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
      'breakpoints' => [],
    ];
  }

  /**
   * Get breakpoints with fallback.
   *
   * @return \Drupal\breakpoint\BreakpointInterface[]
   *   Breakpoints.
   */
  protected function getBreakpoints() {
    $breakpoints = $this->breakpointManager->getBreakpointsByGroup($this->themeManager->getActiveTheme()->getName());
    if (empty($breakpoints)) {
      $breakpoints = $this->breakpointManager->getBreakpointsByGroup('lbl');
    }
    return $breakpoints;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['label']['#access'] = FALSE;

    $configuration = $this->getConfiguration();

    $definitions = $this->getPluginDefinition()->get('variants');
    $breakpoints = $this->getBreakpoints();

    if (count($breakpoints) == 0 || !$definitions || !is_array($definitions) || count($definitions) == 0) {
      return $form;
    }

    $form['#attached']['library'][] = 'lbl/form';
    $form['breakpoints'] = [
      '#type' => 'details',
      '#title' => $this->t('Breakpoints'),
      '#open' => FALSE,
      '#attributes' => ['class' => ['lbl-checkbox-selector']],
    ];
    $renderer = \Drupal::service('renderer');
    /** @var \Drupal\breakpoint\Breakpoint $breakpoint */
    foreach ($breakpoints as $breakpoint) {
      $breakPointPluginId = str_replace('.', '-', $breakpoint->getPluginId());
      $form['breakpoints'][$breakPointPluginId] = [
        '#type' => 'fieldset',
        '#title' => '<h5>' . $breakpoint->getLabel() . '</h5>',
      ];
      $options = [];
      foreach ($definitions as $key => $def) {
        $options[$key] = $renderer->render($this->iconBuilder->build($def['map']));
      }
      $form['breakpoints'][$breakPointPluginId]['config'] = [
        '#type' => 'radios',
        '#options' => $options,
        '#default_value' => ($configuration['breakpoints'][$breakPointPluginId]) ? $configuration['breakpoints'][$breakPointPluginId] : NULL,
      ];
    }
    return $form;
  }

  /**
   * Check if is empty html.
   *
   * @param string $text
   *   Html text to check.definitions
   *   Return true if not empty.
   */
  protected function checkIsNotEmpty($text): bool {
    $t = trim($text);
    $t = strip_tags($t);
    $t = trim($text);
    return $t != "";
  }

  /**
   * {@inheritDoc}
   */
  public function build(array $regions) {
    $build = parent::build($regions);
    $build['#settings']['section_attributes'] = new Attribute();
    $build['#settings']['section_attributes']['class'] = ['lbl-layout'];
    $definitions = $this->getPluginDefinition()->get('variants');
    $breaks = $this->getBreakpoints();

    foreach ($this->getPluginDefinition()->getRegionNames() as $region_name) {
      if (array_key_exists($region_name, $regions)) {
        $build[$region_name]['#attributes']['style'] = 'grid-area: ' . $region_name;
      }
    }

    $defs = array_keys($definitions);

    $id = $this->getPluginDefinition()->id();
    $css = array_reduce($defs, function ($value, $key) use ($definitions, $id) {
      $v = $definitions[$key];
      $map = $v['map'];

      $regions = array_reduce($map, function ($css, $vv) {
        return $css . '"' . implode(" ", $vv) . '" ';
      }, "");

      $cols = array_reduce($map, function ($max, $vv) {
        return (count($vv) > $max) ? count($vv) : $max;
      }, 0);

      $rows = count($map);

      $class = implode('-', [$id, $key]);
      $class = Html::cleanCssIdentifier($class);
      $grid = " { display: grid; grid-template-rows:  repeat($rows, auto); grid-template-areas: $regions; grid-template-columns: repeat($cols, auto) }";

      $value[] = [
        'class' => $class,
        'grid' => $grid,
      ];
      return $value;
    }, []);

    $breakCss = array_map(function ($b) use ($css) {
      $bCss = array_map(function ($v) use ($b) {
        return '.' . $v['class'] . '-' . Html::cleanCssIdentifier($b->getPluginId(), ['.' => '-']) . ' ' . $v['grid'];
      }, $css);
      if ($b->getMediaQuery() == "") {
        return implode("\n", $bCss);
      }
      return '@media ' . $b->getMediaQuery() . ' { ' . implode("\n", $bCss) . '}';
    }, $breaks);
    $build['#attached']['html_head'] = [
      [
        [
          '#tag' => 'style',
          '#value' => implode("\n", $breakCss),
        ],
        $id,
      ],
    ];

    $build['#attached']['library'][] = 'lbl/layout';

    if ($this->configuration['breakpoints']) {
      foreach ($this->configuration['breakpoints'] as $break => $breakpoint) {
        $build['#settings']['section_attributes']['class'][] =
          Html::cleanCssIdentifier(implode('-', [$id, $breakpoint, $break]));
      }
    }
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $breakpoints = $form_state->getValue('breakpoints');
    $configs = [];
    foreach ($breakpoints as $breakpoint => $config) {
      $configs[$breakpoint] = $config['config'];
    }
    $this->configuration['breakpoints'] = $configs;
  }

}
